import json
import os
from codecs import escape_decode
from json.decoder import JSONDecodeError


class Config:
    def __init__(self, fp, data=None):
        self.fp = fp
        self._data = data

    @classmethod
    def from_file(self, fp):
        if not os.path.isfile(fp):
            with open(fp, "w") as f:
                f.write("")
            return Config(fp, {})

        file = open(fp, "r")
        try:
            data = json.load(file)
            return Config(fp, data)
        except JSONDecodeError:
            return Config(fp, {})

        finally:
            file.close()

    @property
    def data(self):
        return self._data if self._data else {}

    def set_values(self, **kwargs):
        for k, v in kwargs.items():
            if not v:
                del self._data[k]
                continue
            self._data[k] = v.encode("utf-8").decode("unicode_escape")

        with open(self.fp, "w") as f:
            json.dump(self._data, f, indent=2)

    def __str__(self):
        return json.dumps(self.data, indent=2)



import sys
import argparse
from unittest.mock import patch

from chromie.parser import parse_args


class ParserHelper:
    @classmethod
    def get_mocked_args(self, cmd):
        argv = self.get_argv(cmd)
        with patch.object(sys, "argv", argv):
            return parse_args(argv=sys.argv)

    @classmethod
    def get_argv(self, cmd):
        return cmd.split()[1:]

    @classmethod
    def parse_args(self, **kwargs):
        return argparse.Namespace(**kwargs)



import unittest
from unittest.mock import patch, Mock

from chromie.commands.init import init
from chromie.enum import Initialize
from chromie.utils import ChromiePathFinder
from tests.utils import ParserHelper


class TestInit(unittest.TestCase):
    @patch("chromie.commands.init.make_extension_dir")
    @patch("chromie.commands.init.ChromiePathFinder.exists", return_value=False)
    def test_init_makes_directory(self, mocked_exists, mocked_make_extension_dir):

        args = ParserHelper.get_mocked_args("chromie init -n testy -o")
        finder = ChromiePathFinder(args.filepath, args.name)
        init(args)

        mocked_make_extension_dir.assert_called_with(finder)

    @patch("chromie.commands.init.ChromiePathFinder.exists", return_value=True)
    @patch("builtins.input", return_value="y")
    def test_init_prompts_for_overwrite(self, mocked_input, mocked_exists):
        args = ParserHelper.get_mocked_args("chromie init -n testy")
        init(args)
        mocked_input.assert_called_with(Initialize.OVERWRIGHT_PROMPT)

    @patch("builtins.input", return_value="testy")
    def test_init_prompts_for_name(self, mocked_input):
        args = ParserHelper.get_mocked_args("chromie init")
        init(args)
        mocked_input.assert_called_with(Initialize.NAME_PROMPT)


if __name__ == "__main__":
    unittest.main()



import unittest
from unittest.mock import patch

from chromie import config


class TestConfig(unittest.TestCase):
    def test_from_file(self):
        pass

    def test_set_values(self):
        pass

    def test_file_does_not_exist(self):
        pass


if __name__ == "__main__":
    unittest.main()



import zipfile
import unittest
import sys
from unittest.mock import patch, MagicMock

from chromie.commands.package import package, is_valid_version
from chromie.enum import Package
from tests.utils import ParserHelper


def call_package(argv):
    args = ParserHelper.get_mocked_args(argv)
    try:
        package(args)
    except:
        pass


class TestPackage(unittest.TestCase):
    def setUp(self):
        self.version = "0.0.1"

    @patch("builtins.input", return_value="patch")
    def test_package_prompts_for_version_type(self, mocked_input):
        call_package("chromie pack")
        mocked_input.assert_called_with(Package.VERSION_PROMPT)

    def test_is_valid_version_true(self):
        valid = is_valid_version(self.version)
        self.assertTrue(valid)

    def test_is_valid_version_false(self):
        valid = is_valid_version("0.0.0.0")
        self.assertFalse(valid)

    @patch("chromie.commands.package.ManifestFile.from_file", return_value=MagicMock())
    @patch("chromie.commands.package.is_valid_version", return_value=True)
    def test_is_valid_version_called(self, mocked_is_valid_version, mocked_from_file):
        call_package(f"chromie pack -v {self.version}")
        mocked_is_valid_version.assert_called_with(self.version)

    @patch("chromie.commands.package.ManifestFile.from_file", return_value=MagicMock())
    def test_manifest_file_version_is_set(self, mocked_from_file):
        call_package(f"chromie pack -v {self.version}")
        mocked_from_file.return_value.set_version.assert_called_with(self.version)


if __name__ == "__main__":
    unittest.main()



import sys

from chromie.parser import parse_args
from chromie.commands import (
    do_init,
    do_pack,
    do_preview,
    do_preview,
    do_config,
    do_upload,
    do_update,
    do_publish,
)


def main(argv=None):
    args = parse_args(argv)

    if args.command == "init":
        do_init(args)

    elif args.command == "pack":
        do_pack(args)

    elif args.command == "preview":
        do_preview(args)

    elif args.command == "config":
        do_config(args)

    elif args.command == "upload":
        do_upload(args)

    elif args.command == "update":
        do_update(args)

    elif args.command == "publish":
        do_publish(args)

    else:
        raise SystemExit("No command selected, for help enter: chromie --help")

    return 0


if __name__ == "__main__":

    sys.exit(main(sys.argv[1:]))



from __future__ import annotations
from typing import Dict
from json.decoder import JSONDecodeError

import requests

from chromie.auth import BaseServiceAuth


class GoogleWebStoreError(Exception):
    pass


class GoogleWebStoreAuthentication(BaseServiceAuth):
    SCOPES = ["https://www.googleapis.com/auth/chromewebstore"]


class GoogleWebStore:
    UPLOAD_URI = "https://www.googleapis.com/upload/chromewebstore/v1.1/items"
    METADATA_URI = "https://www.googleapis.com/chromewebstore/v1.1/items"

    def __init__(
        self, token: Dict[str, str] = None, session: requests.Session = None
    ) -> GoogleWebStore:
        self.token = token
        self._request_session = session if session else requests.Session()

    @staticmethod
    def session(email: str, credentials: Dict[str, str]) -> GoogleWebStoreSession:
        return GoogleWebStoreSession(email, credentials)

    def authenticate(self, email: str, credentials: Dict[str, str]) -> None:
        auth = GoogleWebStoreAuthentication()
        auth.set_credentials(**credentials)
        self.token = auth.get_token(email)
        return None

    def _make_request(self, request: requests.PreparedRequest) -> requests.Response:
        return self._request_session.send(request)

    def _build_request(
        self, method: str, url: str, fp: str = None
    ) -> requests.PreparedRequest:
        request = requests.Request(method=method, url=url)
        headers = {
            "Authorization": "Bearer {}".format(self.token),
            "x-goog-api-version": "2",
        }
        if fp:
            with open(fp, "rb") as file:
                request.data = file.read()
                request.params = {"file": fp}
                headers["Accept-Encoding"] = "gzip, deflate"
        request.headers = headers
        return request.prepare()

    def _handle_response(self, response: requests.Response) -> str:
        try:
            data = response.json()
            if not response.ok:
                errors = data.get("error").get("message").split(";")
                if len(errors) == 1:
                    raise GoogleWebStoreError(errors[0])
                message = "\n  " + "\n  ".join(
                    [
                        f"{i}. {err.split(':')[1].strip()}"
                        if ":" in err and "http" not in err
                        else f"{i}. {err.strip()}"
                        for i, err in enumerate(errors, 1)
                    ]
                )
                raise GoogleWebStoreError(message)
            elif response.ok and data.get("uploadState") == "FAILURE":
                errors = data.get("itemError")
                message = f"\n" + "\n".join(
                    [
                        f"{i}. {err.get('error_detail')}"
                        for i, err in enumerate(errors, 1)
                    ]
                )
                raise GoogleWebStoreError(message)
            elif response.ok and data.get("uploadState") == "SUCCESS":
                return data.get("id")
        except JSONDecodeError:
            raise GoogleWebStoreError(response.text)

    def upload(self, filepath: str) -> str:
        request = self._build_request("POST", url=self.UPLOAD_URI, fp=filepath)
        response = self._make_request(request)
        return self._handle_response(response)

    def publish(self, id: str) -> str:
        url = "/".join([self.METADATA_URI, id, "publish"])
        request = self._build_request(method="POST", url=url)
        response = self._make_request(request)
        return self._handle_response(response)

    def update(self, id: str, filepath: str) -> str:
        url = "/".join([self.UPLOAD_URI, id])
        request = self._build_request(method="PUT", url=url, fp=filepath)
        response = self._make_request(request)
        return self._handle_response(response)


class GoogleWebStoreSession:
    def __init__(
        self, email: str, credentials: Dict[str, str], session: requests.Session = None
    ) -> GoogleWebStoreSession:
        self.email = email
        self.credentials = credentials
        self.session = session
        self.store = None

    def __enter__(self) -> GoogleWebStore:
        self.store = GoogleWebStore(session=self.session)
        self.store.authenticate(self.email, self.credentials)
        return self.store

    def __exit__(self, *args):
        return None



import argparse

from chromie.__version__ import __version__


def parse_args(argv=None):
    parser = argparse.ArgumentParser(prog="chromie")

    parser.add_argument(
        "-v", "--version", action="version", version=f"chromie-{__version__}"
    )

    subparsers = parser.add_subparsers(
        help="desired command to perform", dest="command"
    )

    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "filepath",
        help="project directory",
        nargs="?",
        action="store",
        type=str,
        default=".",
    )

    init_parser = subparsers.add_parser(
        "init", parents=[parent_parser], help="initialize project directory"
    )

    init_parser.add_argument(
        "-n", "--name", help="specify the project name", action="store", type=str
    )

    init_parser.add_argument(
        "-o",
        "--overwrite",
        help="overwrite files if directory already exists",
        action="store_true",
    )

    package_parser = subparsers.add_parser(
        "pack", parents=[parent_parser], help="package project directory"
    )

    package_parser.add_argument(
        "-i",
        "--increment-version",
        help="increment project version before packaging based with (major, minor, patch) as options",
        action="store",
    )

    config_parser = subparsers.add_parser(
        "config", parents=[parent_parser], help="manage config file"
    )

    config_parser.add_argument("name", action="store", type=str)

    config_parser.add_argument("value", action="store", type=str)

    package_parser.add_argument(
        "-v", "--version", help="specify the version", action="store"
    )

    upload_parser = subparsers.add_parser(
        "upload", parents=[parent_parser], help="upload project to web store"
    )

    update_parser = subparsers.add_parser(
        "update", parents=[parent_parser], help="push updates to web store"
    )

    publish_parser = subparsers.add_parser(
        "publish", parents=[parent_parser], help="publish to web store"
    )

    preview_parser = subparsers.add_parser(
        "preview", parents=[parent_parser], help="preview project in browser"
    )
    return parser.parse_args(argv)



import argparse
import unittest
from unittest.mock import patch, Mock

from chromie import main
from tests.utils import ParserHelper


class TestMain(unittest.TestCase):
    def test_without_arguments(self):
        with self.assertRaises(SystemExit):
            main([])

    @patch("chromie.do_init")
    def test_init(self, init):
        cmd = "chromie init -n testy"
        argv = ParserHelper.get_argv(cmd)
        mocked_args = ParserHelper.parse_args(
            command="init", filepath=".", name="testy", overwrite=False
        )
        main(argv)

        init.assert_called_with(mocked_args)

    @patch("chromie.do_pack")
    def test_pack(self, package):
        cmd = "chromie pack ./testy"
        argv = ParserHelper.get_argv(cmd)
        mocked_args = ParserHelper.parse_args(
            command="pack", filepath="./testy", increment_version=None, version=None
        )
        main(argv)

        package.assert_called_with(mocked_args)

    def test_with_invalid_command(self):
        with self.assertRaises(SystemExit):
            main(["thisisnotarealcommand"])


if __name__ == "__main__":
    unittest.main()



import os

from chromie.config import Config


def config(args):
    filepath = os.path.abspath(args.filepath)
    dot_chromie = os.path.join(filepath, ".chromie")

    if not os.path.isdir(dot_chromie):
        os.mkdir(dot_chromie)

    config_file = Config.from_file(os.path.join(filepath, ".chromie/settings.json"))
    data = {args.name: args.value}
    config_file.set_values(**data)



import argparse
import unittest

from chromie.parser import parse_args
from tests.utils import ParserHelper


class TestArgumentParser(unittest.TestCase):
    def test_init_required_arguments(self):
        args = ParserHelper.get_mocked_args("chromie init . -n testy")
        self.assertEqual(args.name, "testy")
        self.assertEqual(args.command, "init")
        self.assertEqual(args.filepath, ".")

    def test_init_overwrite(self):
        args = ParserHelper.get_mocked_args("chromie init . -o")
        self.assertTrue(args.overwrite)

    def test_init_name(self):
        args = ParserHelper.get_mocked_args("chromie init . -n testy")
        self.assertEqual(args.name, "testy")

    def test_pack_required_arguments(self):
        args = ParserHelper.get_mocked_args("chromie pack ./testy")
        self.assertEqual(args.command, "pack")
        self.assertEqual(args.filepath, "./testy")

    def test_pack_args_specific_version(self):
        args = ParserHelper.get_mocked_args("chromie pack ./testy -v 0.0.8")
        self.assertEqual(args.version, "0.0.8")

    def test_pack_args_increment_version(self):
        args = ParserHelper.get_mocked_args("chromie pack ./testy -i patch")
        self.assertEqual(args.increment_version, "patch")


if __name__ == "__main__":
    unittest.main()



import subprocess
import os

from chromie.utils import ChromiePathFinder


def find_chrome():
    chrome = "Google Chrome.app"
    if os.name == "posix":
        with subprocess.Popen(["mdfind", chrome], stdout=subprocess.PIPE) as proc:
            results = [
                line
                for line in proc.stdout.read().decode("utf-8").split("\n")
                if line.endswith(chrome)
            ]
            if results:
                return results[0]
            None
    else:
        raise NotImplemented("This function has only been developed for macOS.")


def preview(args):
    finder = ChromiePathFinder(args.filepath)
    try:
        chrome = find_chrome()
        subprocess.call(
            [
                f"{chrome}/Contents/MacOS/Google Chrome",
                f'--load-extension={finder("src")}',
            ]
        )
    except KeyboardInterrupt as e:
        exit()



from typing import List, Dict
import time

import requests
import jwt


class AuthenticationError(Exception):
    pass


class BaseServiceAuth:
    TOKEN_URI = "https://oauth2.googleapis.com/token"
    SCOPES = []

    def __init__(self, credentials: Dict[str, str] = None):
        self._creds = credentials if credentials else {}

    @classmethod
    def add_scopes(cls, scopes: List[str]) -> None:
        cls.SCOPES.extend(scopes)

    def get_token(self, email: str) -> str:
        data = {
            "grant_type": "urn:ietf:params:oauth:grant-type:jwt-bearer",
            "assertion": self._encode_web_token(email),
        }
        response = requests.post(
            self.TOKEN_URI,
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            data=data,
        )
        return response.json().get("access_token")

    def _encode_web_token(self, email: str) -> str:
        try:
            iat = round(time.time())
            exp = iat + 60
            payload = {
                "iss": self._creds.get("client_email", None),
                "sub": email,
                "scope": " ".join(self.SCOPES) if len(self.SCOPES) > 0 else "",
                "aud": self.TOKEN_URI,
                "exp": exp,
                "iat": iat,
            }
            return jwt.encode(
                payload, self._creds.get("private_key", None), algorithm="RS256"
            )
        except TypeError as e:
            raise AuthenticationError(e)

    def set_credentials(self, **kwargs: str) -> None:
        for k, v in kwargs.items():
            self._creds[k] = v



import shutil
import os

from chromie.utils import ChromiePathFinder, ManifestFile
from chromie.enum import Initialize, Path


def make_extension_dir(finder):

    os.makedirs(finder(Path.STORE_DIR))
    os.makedirs(finder(Path.IMAGES_DIR))

    with open(finder(Path.IGNORE_FILE), "w") as f:
        f.write("")

    ManifestFile(
        finder(Path.MANIFEST_FILE),
        {"name": finder.name, "manifest_version": 2, "version": "0.0.0"},
    ).write()


def init(args):

    name = args.name if args.name else input(Initialize.NAME_PROMPT)
    overwrite = args.overwrite

    finder = ChromiePathFinder(args.filepath, name)

    if not overwrite and finder.exists() == True:
        asked = 0
        overwrite_prompt = ""
        while asked <= 3 or overwrite_prompt not in [
            *Initialize.AFFERMATIVE,
            *Initialize.NEGATIVE,
        ]:
            overwrite_prompt = input(Initialize.OVERWRIGHT_PROMPT).upper()
            asked += 1
            if overwrite_prompt in Initialize.NEGATIVE or asked >= 3:
                raise SystemExit()

            elif overwrite_prompt in Initialize.AFFERMATIVE:
                shutil.rmtree(finder.root, ignore_errors=True)
                break

    elif overwrite == True:
        shutil.rmtree(finder.root, ignore_errors=True)

    if not finder.exists(finder.root):
        make_extension_dir(finder)



import os
import re
from glob import glob
import zipfile

from chromie.utils import ChromiePathFinder, ManifestFile
from chromie.enum import Package, Path


def is_valid_increment_version(increment_version):
    if increment_version and increment_version.lower() in ("minor", "major", "patch"):
        return True
    return False


def is_valid_version(version):
    pattern = re.compile(
        r"^([0-9]+)\.([0-9]+)\.([0-9]+)(?:-([0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*))?(?:\+[0-9A-Za-z-]+)?$"
    )
    return True if version and pattern.match(version) else False


def write_zip(zip, fp, root, name):
    path = os.path.abspath(os.path.join(root, name))
    if os.name == "posix":
        arcname = f"{fp.split('/')[-1]}/{name}"
        zip.write(path, arcname)
    else:
        zip.write(path, os.path.basename(path))


def package_directory(fp, src, target, ignore_paths=None):
    ignore_paths = [] if not ignore_paths else ignore_paths

    with zipfile.ZipFile(target, "w", zipfile.ZIP_STORED) as zip:
        for dirname, subdirs, files in os.walk(src):
            [
                write_zip(zip, fp, dirname, fn)
                for fn in files
                if os.path.join(dirname, fn) not in ignore_paths
            ]
            [
                write_zip(zip, fp, dirname, dn)
                for dn in subdirs
                if os.path.join(dirname, dn) not in ignore_paths
            ]


def package(args):

    filepath = os.path.abspath(args.filepath)
    increment_version = args.increment_version
    version = args.version

    if not version and not increment_version:
        increment_version = input(Package.VERSION_PROMPT)

    finder = ChromiePathFinder(filepath)

    manifest_file = ManifestFile.from_file(finder(Path.MANIFEST_FILE))

    if increment_version and not version:
        if not is_valid_increment_version(increment_version):
            raise SystemExit(Package.INVALID_VERSION_ARGUMENT)
        version = manifest_file.increment_version(increment_version)

    elif version and not increment_version:
        if not is_valid_version(version):
            raise SystemExit(Package.INVALID_VERSION_PATTERN)
        manifest_file.set_version(version)

    dist = finder(Path.DIST_DIR)
    if not os.path.isdir(dist):
        os.mkdir(dist)

    src = finder(Path.SRC_DIR)
    with open(finder(Path.IGNORE_FILE), "r") as zipignore:
        ignore_paths = [
            name
            for pattern in [line.rstrip() for line in zipignore.readlines()]
            for name in glob(os.path.join(filepath, src, pattern))
        ]

        package_directory(
            filepath,
            os.path.join(filepath, src),
            os.path.join(dist, f"{finder.name}-{version}.zip"),
            ignore_paths,
        )


if __name__ == "__main__":
    print(Path.MANIFEST_FILE)



from enum import Enum


class Path:
    IGNORE_FILE = ".zipignore"
    DIST_DIR = "dist/"
    STORE_DIR = "dist/web store/"
    SRC_DIR = "src/"
    IMAGES_DIR = "src/images/"
    MANIFEST_FILE = "src/manifest.json"


class Settings:
    MISSING = "Please create a '.chromie/settings.json' file within the root project directory"
    INVALID = "There was an error authenticating. Please check that the credentials provided in '.chromie/settings.json are valid."


class Publish:
    SUCCESS = "Successfully published to the Google Chrome Web Store."
    UNSUCCESSFUL = (
        "Publishing this chrome extension was unsuccessful for the following reasons: "
    )


class Update:
    SUCCESS = "Update Successful"


class Package:

    VERSION_PROMPT = (
        ">> How would you like to increment the version?\n"
        ">> Options are either 'major', 'minor', or 'patch':\n"
    )
    INVALID_VERSION_ARGUMENT = "Incorrect version was entered. Please enter either 'major', 'minor' or 'patch'."
    INVALID_VERSION_PATTERN = (
        "The version provided was not valid. Please see https://semver.org/ for help."
    )
    NO_DIST_FOUND = "No dist directory was found in the directory specified."
    ZIP_SUCCESSFUL = "{} was packaged successfully!"


class Initialize:
    NAME_PROMPT = ">> What is the name of your project?\n "
    OVERWRIGHT_PROMPT = (
        ">> This directory with this name already exists.\n"
        ">> Would you like to overwrite? Y/N:\n"
    )
    AFFERMATIVE = (
        "Y",
        "YES",
    )
    NEGATIVE = (
        "N",
        "NO",
    )



import os
from zipfile import is_zipfile

from chromie.config import Config
from chromie.webstore import GoogleWebStore, GoogleWebStoreError
from chromie.auth import AuthenticationError
from chromie.enum import Settings, Update


def get_latest_version(fp):
    archives = [fn for fn in os.listdir(fp) if is_zipfile(os.path.join(fp, fn))]

    archives.sort(reverse=True)
    return os.path.join(fp, archives[0])


def update(args):
    root = os.path.abspath(args.filepath)

    dist = os.path.join(root, "dist")
    dot_chromie = os.path.join(root, ".chromie")

    if not os.path.isdir(dot_chromie):
        raise SystemExit(Settings.MISSING)

    archive = get_latest_version(dist)

    config_file = Config.from_file(os.path.join(dot_chromie, "settings.json"))

    try:
        with GoogleWebStore.session(
            config_file.data.get("email"), credentials=config_file.data
        ) as session:
            try:
                extension_id = session.update(config_file.data["extension_id"], archive)
                print(Update.SUCCESS)
            except GoogleWebStoreError as e:
                print(e)
    except AuthenticationError:
        print(Settings.INVALID)



from setuptools import setup, Command
import os
import sys

here = os.path.abspath(os.path.dirname(__file__))

with open("README.md", "r") as f:
    readme = f.read()

about = {}
with open(os.path.join(here, "chromie", "__version__.py"), "r") as f:
    exec(f.read(), about)

packages = ["chromie"]

requires = ["requests>=2.24.0,<3", "cryptography>=3.0,<4", "PyJWT>=1.7.1,<2"]


class Publish(Command):
    description = "Publish to PyPI"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system("python setup.py sdist bdist_wheel")
        os.system("twine upload dist/* --skip-existing")


setup(
    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=readme,
    packages=packages,
    long_description_content_type="text/markdown",
    url="https://github.com/Sandersland/chromie",
    author=about["__author__"],
    author_email=about["__email__"],
    license=about["__license__"],
    extras_require={"dev": ["twine", "black"]},
    install_requires=requires,
    entry_points={"console_scripts": ["chromie=chromie:main"]},
    cmdclass={"publish": Publish},
)



from chromie.commands.init import init as do_init
from chromie.commands.package import package as do_pack
from chromie.commands.preview import preview as do_preview
from chromie.commands.config import config as do_config
from chromie.commands.upload import upload as do_upload
from chromie.commands.update import update as do_update
from chromie.commands.publish import publish as do_publish
